package com.nisum.employeeservice.controller;

import com.nisum.employeeservice.exception.ResourceNotFoundException;
import com.nisum.employeeservice.model.Employee;
import com.nisum.employeeservice.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/v1")
public class EmployeeController {

  @Autowired EmployeeService employeeService;

  // Return all employees
  // GET: http://localhost:8080/api/v1/employees/
  @GetMapping("/employees")
  public List<Employee> getAllEmployees() {
    return employeeService.findAllEmployees();
  }

  // Create a new employee
  // POST: http://localhost:8080/api/v1/employees/create
  // send this in postman: {"id": "4", "firstName": "Kyle", "lastName": "Muldoon"}
  @PostMapping("/employees/create")
  public Employee createEmployee(@RequestBody Employee employee) {
    return employeeService.addEmployee(employee);
  }

  // Get a particular employee
  // GET: http://localhost:8080/api/v1/employees/__num__
  @GetMapping("/employees/{id}")
  public ResponseEntity<Employee> getEmployeeById(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
        .orElseThrow(() -> new ResourceNotFoundException("Employee by Id Not Found" + employeeId));
    return ResponseEntity.ok().body(employee);
  }

  // Update a particular employee
  @PutMapping("/employees/update/{id}")
  public ResponseEntity<Employee> updateEmployeeById(@PathVariable(value = "id") Long employeeId, @RequestBody Employee employeeNew) throws ResourceNotFoundException {
    Employee employeeOld = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee by Id Not Found" + employeeId));

    employeeService.deleteEmployeeById(employeeOld.getId());
    employeeService.addEmployee(employeeNew);

    return ResponseEntity.ok().body(employeeNew);
  }


  // Delete a particular employee
  @DeleteMapping("/employees/remove/{id}")
  public ResponseEntity<Employee> deleteEmployeeById(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {

    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee by Id Not Found" + employeeId));
    employeeService.deleteEmployeeById(employee.getId());

    return ResponseEntity.ok().body(employee);
  }

}
